@@ -29,8 +29,7 @@
 #include "silc-channels.h"
 #include "silc-queries.h"
 #include "silc-nicklist.h"
-#include "version_internal.h"
-#include "version.h"
+#include "silcversion.h"
 
 #include "signals.h"
 #include "levels.h"
@@ -42,28 +41,18 @@
 
 /* Command line option variables */
 static bool opt_create_keypair = FALSE;
-static bool opt_list_ciphers = FALSE;
-static bool opt_list_hash = FALSE;
-static bool opt_list_hmac = FALSE;
-static bool opt_list_pkcs = FALSE;
-static bool opt_version = FALSE;
-static char *opt_debug = FALSE;
 static char *opt_pkcs = NULL;
-static char *opt_keyfile = NULL;
 static int opt_bits = 0;
 
 static int idletag;
 
 SilcClient silc_client = NULL;
-SilcClientConfig silc_config = NULL;
 extern SilcClientOperations ops;
 extern bool silc_debug;
 extern bool silc_debug_hexdump;
-#ifdef SILC_SIM
-/* SIM (SILC Module) table */
-SilcSimContext **sims = NULL;
-uint32 sims_count = 0;
-#endif
+
+void silc_expandos_init(void);
+void silc_expandos_deinit(void);
 
 static int my_silc_scheduler(void)
 {
@@ -91,6 +80,11 @@ static SERVER_CONNECT_REC *create_server_connect(void)
   return g_malloc0(sizeof(SILC_SERVER_CONNECT_REC));
 }
 
+static void destroy_server_connect(SERVER_CONNECT_REC *conn)
+{
+
+}
+
 /* Checks user information and saves them to the config file it they
    do not exist there already. */
 
@@ -106,7 +100,7 @@ static void silc_init_userinfo(void)
     if (!str)
       str = g_getenv("IRCNAME");
     settings_set_str("real_name",
-		     str != NULL ? str : g_get_real_name());
+		     str != NULL ? str : silc_get_real_name());
   }
  
   /* username */
@@ -116,8 +110,8 @@ static void silc_init_userinfo(void)
     if (!str)
       str = g_getenv("IRCUSER");
     settings_set_str("user_name",
-		     str != NULL ? str : g_get_user_name());
-    
+		     str != NULL ? str : silc_get_username());
+
     user_name = settings_get_str("user_name");
   }
 
@@ -135,12 +129,7 @@ static void silc_init_userinfo(void)
   /* alternate nick */
   set = settings_get_str("alternate_nick");
   if (set == NULL || *set == '\0') {
-    if (strlen(nick) < 9)
-      str = g_strconcat(nick, "_", NULL);
-    else { 
-      str = g_strdup(nick);
-      str[strlen(str)-1] = '_';
-    }
+    str = g_strconcat(nick, "_", NULL);
     settings_set_str("alternate_nick", str);
     g_free(str);
   }
@@ -156,59 +145,165 @@ static void silc_init_userinfo(void)
   }
 }
 
+#ifdef SILC_DEBUG
+static bool i_debug;
+static bool silc_irssi_debug_print(char *file, char *function, int line,
+				   char *message, void *context)
+{
+  printtext(NULL, NULL, MSGLEVEL_CLIENTCRAP,
+	    "DEBUG: %s:%d: %s", function, line, message);
+  return TRUE;
+}
+
+static void sig_debug_setup_changed(void)
+{
+  bool debug = settings_get_bool("debug");
+  if (debug) {
+    const char *debug_string = settings_get_str("debug_string");
+    i_debug = silc_debug = TRUE;
+    if (strlen(debug_string))
+      silc_log_set_debug_string(debug_string);
+    silc_log_set_debug_callbacks(silc_irssi_debug_print, NULL, NULL, NULL);
+    return;
+  }
+  if (i_debug)
+    silc_debug = FALSE;
+}
+#endif
+
 /* Log callbacks */
 
 static bool silc_log_misc(SilcLogType type, char *message, void *context)
 {
-  fprintf(stderr, "%s\n", message);
+  printtext(NULL, NULL, MSGLEVEL_CLIENTCRAP, "%s: %s", 
+	    (type == SILC_LOG_INFO ? "[Info]" :
+	     type == SILC_LOG_WARNING ? "[Warning]" : "[Error]"), message);
   return TRUE;
 }
 
-/* Init SILC. Called from src/fe-text/silc.c */
-
-void silc_core_init(void)
-{
-  static struct poptOption options[] = {
-    { "create-key-pair", 'C', POPT_ARG_NONE, &opt_create_keypair, 0, 
-      "Create new public key pair", NULL },
-    { "pkcs", 0, POPT_ARG_STRING, &opt_pkcs, 0, 
-      "Set the PKCS of the public key pair", "PKCS" },
-    { "bits", 0, POPT_ARG_INT, &opt_bits, 0, 
-      "Set the length of the public key pair", "VALUE" },
-    { "show-key", 'S', POPT_ARG_STRING, &opt_keyfile, 0, 
-      "Show the contents of the public key", "FILE" },
-    { "list-ciphers", 'C', POPT_ARG_NONE, &opt_list_ciphers, 0,
-      "List supported ciphers", NULL },
-    { "list-hash-funcs", 'H', POPT_ARG_NONE, &opt_list_hash, 0,
-      "List supported hash functions", NULL },
-    { "list-hmacs", 'H', POPT_ARG_NONE, &opt_list_hmac, 0,
-      "List supported HMACs", NULL },
-    { "list-pkcs", 'P', POPT_ARG_NONE, &opt_list_pkcs, 0,
-      "List supported PKCSs", NULL },
-    { "debug", 'd', POPT_ARG_STRING, &opt_debug, 0,
-      "Enable debugging", "STRING" },
-    { "version", 'V', POPT_ARG_NONE, &opt_version, 0,
-      "Show version", NULL },
-    { NULL, '\0', 0, NULL }
-  };
-
-  args_register(options);
-}
-
 static void silc_nickname_format_parse(const char *nickname,
 				       char **ret_nickname)
 {
   silc_parse_userfqdn(nickname, ret_nickname, NULL);
 }
 
-/* Finalize init. Called from src/fe-text/silc.c */
-
-void silc_core_init_finish(void)
+static void silc_register_cipher(SilcClient client, const char *cipher)
 {
-  CHAT_PROTOCOL_REC *rec;
-  SilcClientParams params;
+  int i;
 
-  if (opt_create_keypair == TRUE) {
+  if (cipher) {
+    for (i = 0; silc_default_ciphers[i].name; i++)
+      if (!strcmp(silc_default_ciphers[i].name, cipher)) {
+	silc_cipher_register(&(silc_default_ciphers[i]));
+	break;
+      }
+    
+    if (!silc_cipher_is_supported(cipher)) {
+      SILC_LOG_ERROR(("Unknown cipher `%s'", cipher));
+      exit(1);
+    }
+  }
+
+  /* Register other defaults */
+  silc_cipher_register_default();
+}
+
+static void silc_register_hash(SilcClient client, const char *hash)
+{
+  int i;
+
+  if (hash) {
+    for (i = 0; silc_default_hash[i].name; i++)
+      if (!strcmp(silc_default_hash[i].name, hash)) {
+	silc_hash_register(&(silc_default_hash[i]));
+	break;
+      }
+    
+    if (!silc_hash_is_supported(hash)) {
+      SILC_LOG_ERROR(("Unknown hash function `%s'", hash));
+      exit(1);
+    }
+  }
+
+  /* Register other defaults */
+  silc_hash_register_default();
+}
+
+static void silc_register_hmac(SilcClient client, const char *hmac)
+{
+  int i;
+
+  if (hmac) {
+    for (i = 0; silc_default_hmacs[i].name; i++)
+      if (!strcmp(silc_default_hmacs[i].name, hmac)) {
+	silc_hmac_register(&(silc_default_hmacs[i]));
+	break;
+      }
+    
+    if (!silc_hmac_is_supported(hmac)) {
+      SILC_LOG_ERROR(("Unknown HMAC `%s'", hmac));
+      exit(1);
+    }
+  }
+
+  /* Register other defaults */
+  silc_hmac_register_default();
+}
+
+/* Finalize init. Init finish signal calls this. */
+
+void silc_opt_callback(poptContext con, 
+		       enum poptCallbackReason reason,
+		       const struct poptOption *opt,
+		       const char *arg, void *data)
+{
+  if (strcmp(opt->longName, "show-key") == 0) {
+    /* Dump the key */
+    silc_cipher_register_default();
+    silc_pkcs_register_default();
+    silc_hash_register_default();
+    silc_hmac_register_default();
+    silc_client_show_key((char *)arg);
+    exit(0);
+  }
+
+  if (strcmp(opt->longName, "list-ciphers") == 0) {
+    silc_cipher_register_default();
+    silc_client_list_ciphers();
+    exit(0);
+  }
+
+  if (strcmp(opt->longName, "list-hash-funcs") == 0) {
+    silc_hash_register_default();
+    silc_client_list_hash_funcs();
+    exit(0);
+  }
+
+  if (strcmp(opt->longName, "list-hmacs") == 0) {
+    silc_hmac_register_default();
+    silc_client_list_hmacs();
+    exit(0);
+  }
+
+  if (strcmp(opt->longName, "list-pkcs") == 0) {
+    silc_pkcs_register_default();
+    silc_client_list_pkcs();
+    exit(0);
+  }
+
+  if (strcmp(opt->longName, "debug") == 0) {
+    silc_debug = TRUE;
+    silc_debug_hexdump = TRUE;
+    silc_log_set_debug_string(arg);
+#ifndef SILC_DEBUG
+    fprintf(stdout, 
+	    "Run-time debugging is not enabled. To enable it recompile\n"
+	    "the client with --enable-debug configuration option.\n");
+    sleep(1);
+#endif
+  }
+
+  if (strcmp(opt->longName, "create-key-pair") == 0) {
     /* Create new key pair and exit */
     silc_cipher_register_default();
     silc_pkcs_register_default();
@@ -218,87 +313,81 @@ void silc_core_init_finish(void)
 				NULL, NULL, NULL, NULL, NULL);
     exit(0);
   }
+}
 
-  if (opt_keyfile) {
-    /* Dump the key */
-    silc_cipher_register_default();
-    silc_pkcs_register_default();
-    silc_hash_register_default();
-    silc_hmac_register_default();
-    silc_client_show_key(opt_keyfile);
-    exit(0);
-  }
+/* Init SILC. Called from src/fe-text/silc.c */
 
-  if (opt_list_ciphers) {
-    silc_cipher_register_default();
-    silc_client_list_ciphers();
-    exit(0);
-  }
+void silc_core_init(void)
+{
+  static struct poptOption silc_options[] = {
+    { NULL, '\0', POPT_ARG_CALLBACK, (void *)&silc_opt_callback, '\0', NULL },
+    { "show-key", 'S', POPT_ARG_STRING, NULL, 0,
+      "Show the contents of the public key", "FILE" },
+    { "list-ciphers", 0, POPT_ARG_NONE, NULL, 0,
+      "List supported ciphers", NULL },
+    { "list-hash-funcs", 0, POPT_ARG_NONE, NULL, 0,
+      "List supported hash functions", NULL },
+    { "list-hmacs", 0, POPT_ARG_NONE, NULL, 0,
+      "List supported HMACs", NULL },
+    { "list-pkcs", 0, POPT_ARG_NONE, NULL, 0,
+      "List supported PKCSs", NULL },
+    { "debug", 'd', POPT_ARG_STRING, NULL, 0,
+      "Enable debugging", "STRING" },
+    { "create-key-pair", 'C', POPT_ARG_NONE, NULL, 0,
+      "Create new public key pair", NULL },
+    { "pkcs", 0, POPT_ARG_STRING, &opt_pkcs, 0,
+      "Set the PKCS of the public key pair (-C)", "PKCS" },
+    { "bits", 0, POPT_ARG_INT, &opt_bits, 0,
+      "Set the length of the public key pair (-C)", "VALUE" },
+    { NULL, '\0', 0, NULL }
+  };
 
-  if (opt_list_hash) {
-    silc_hash_register_default();
-    silc_client_list_hash_funcs();
-    exit(0);
-  }
+  CHAT_PROTOCOL_REC *rec;
+  SilcClientParams params;
+  const char *def_cipher, *def_hash, *def_hmac;
 
-  if (opt_list_hmac) {
-    silc_hmac_register_default();
-    silc_client_list_hmacs();
-    exit(0);
-  }
+  args_register(silc_options);
 
-  if (opt_list_pkcs) {
-    silc_pkcs_register_default();
-    silc_client_list_pkcs();
-    exit(0);
-  }
-
-  if (opt_version) {
-    printf("SILC Secure Internet Live Conferencing, version %s "
-	   "(base: SILC Toolkit %s)\n", silc_dist_version, silc_version);
-    printf("(c) 1997 - 2001 Pekka Riikonen <priikone@silcnet.org>\n");
-    exit(0); 
-  }
-
-  if (opt_debug) {
-    silc_debug = TRUE;
-    silc_debug_hexdump = TRUE;
-    silc_log_set_debug_string(opt_debug);
-    silc_log_set_callback(SILC_LOG_INFO, silc_log_misc, NULL);
-    silc_log_set_callback(SILC_LOG_WARNING, silc_log_misc, NULL);
-    silc_log_set_callback(SILC_LOG_ERROR, silc_log_misc, NULL);
-    silc_log_set_callback(SILC_LOG_FATAL, silc_log_misc, NULL);
-#ifndef SILC_DEBUG
-    fprintf(stdout, 
-	    "Run-time debugging is not enabled. To enable it recompile\n"
-	    "the client with --enable-debug configuration option.\n");
-#endif
-  }
-
-  /* Do some irssi initializing */
+  /* Settings */
   settings_add_bool("server", "skip_motd", FALSE);
   settings_add_str("server", "alternate_nick", NULL);
-  
-  /* Initialize the auto_addr variables Is "server" the best choice for
-   * this?  No existing category seems to apply.
-   */
   settings_add_bool("server", "use_auto_addr", FALSE);
   settings_add_str("server", "auto_bind_ip", "");
   settings_add_str("server", "auto_public_ip", "");
   settings_add_int("server", "auto_bind_port", 0);
-        	    	    	
+  settings_add_str("server", "crypto_default_cipher", SILC_DEFAULT_CIPHER);
+  settings_add_str("server", "crypto_default_hash", SILC_DEFAULT_HASH);
+  settings_add_str("server", "crypto_default_hmac", SILC_DEFAULT_HMAC);
+  settings_add_int("server", "key_exchange_timeout_secs", 120);
+  settings_add_int("server", "key_exchange_rekey_secs", 3600);
+  settings_add_int("server", "connauth_request_secs", 2);
+
+#ifdef SILC_DEBUG
+  settings_add_bool("debug", "debug", FALSE);
+  settings_add_str("debug", "debug_string", "");
+  signal_add("setup changed", (SIGNAL_FUNC) sig_debug_setup_changed);
+#endif
+
   silc_init_userinfo();
 
   /* Initialize client parameters */
   memset(&params, 0, sizeof(params));
   strcat(params.nickname_format, "%n@%h%a");
   params.nickname_parse = silc_nickname_format_parse;
+  params.rekey_secs = settings_get_int("key_exchange_rekey_secs");
+  params.connauth_request_secs = settings_get_int("connauth_request_secs");
 
   /* Allocate SILC client */
   silc_client = silc_client_alloc(&ops, &params, NULL, silc_version_string);
 
-  /* Load local config file */
-  silc_config = silc_client_config_alloc(SILC_CLIENT_HOME_CONFIG_FILE);
+  /* Get the ciphers and stuff from config file */
+  def_cipher = settings_get_str("crypto_default_cipher");
+  def_hash = settings_get_str("crypto_default_hash");
+  def_hmac = settings_get_str("crypto_default_hmac");
+  silc_register_cipher(silc_client, def_cipher);
+  silc_register_hash(silc_client, def_hash);
+  silc_register_hmac(silc_client, def_hmac);
+  silc_pkcs_register_default();
 
   /* Get user information */
   silc_client->username = g_strdup(settings_get_str("user_name"));
@@ -306,25 +395,6 @@ void silc_core_init_finish(void)
   silc_client->hostname = silc_net_localhost();
   silc_client->realname = g_strdup(settings_get_str("real_name"));
 
-  /* Register all configured ciphers, PKCS and hash functions. */
-  if (silc_config) {
-    silc_config->client = silc_client;
-    if (!silc_client_config_register_ciphers(silc_config))
-      silc_cipher_register_default();
-    if (!silc_client_config_register_pkcs(silc_config))
-      silc_pkcs_register_default();
-    if (!silc_client_config_register_hashfuncs(silc_config))
-      silc_hash_register_default();
-    if (!silc_client_config_register_hmacs(silc_config))
-      silc_hmac_register_default();
-  } else {
-    /* Register default ciphers, pkcs, hash funtions and hmacs. */
-    silc_cipher_register_default();
-    silc_pkcs_register_default();
-    silc_hash_register_default();
-    silc_hmac_register_default();
-  }
-
   /* Check ~/.silc directory and public and private keys */
   if (silc_client_check_silc_dir() == FALSE) {
     idletag = -1;
@@ -343,6 +413,11 @@ void silc_core_init_finish(void)
     return;
   }
 
+  silc_log_set_callback(SILC_LOG_INFO, silc_log_misc, NULL);
+  silc_log_set_callback(SILC_LOG_WARNING, silc_log_misc, NULL);
+  silc_log_set_callback(SILC_LOG_ERROR, silc_log_misc, NULL);
+  silc_log_set_callback(SILC_LOG_FATAL, silc_log_misc, NULL);
+
   /* Register SILC to the irssi */
   rec = g_new0(CHAT_PROTOCOL_REC, 1);
   rec->name = "SILC";
@@ -352,6 +427,7 @@ void silc_core_init_finish(void)
   rec->create_server_setup = create_server_setup;
   rec->create_channel_setup = create_channel_setup;
   rec->create_server_connect = create_server_connect;
+  rec->destroy_server_connect = destroy_server_connect;
   rec->server_connect = (SERVER_REC *(*) (SERVER_CONNECT_REC *))
     silc_server_connect; 
   rec->channel_create = (CHANNEL_REC *(*) (SERVER_REC *, const char *, int))
@@ -365,8 +441,11 @@ void silc_core_init_finish(void)
   silc_server_init();
   silc_channels_init();
   silc_queries_init();
+  silc_expandos_init();
 
   idletag = g_timeout_add(5, (GSourceFunc) my_silc_scheduler, NULL);
+
+  module_register("silc", "core");
 }
 
 /* Deinit SILC. Called from src/fe-text/silc.c */
@@ -376,10 +455,14 @@ void silc_core_deinit(void)
   if (idletag != -1) {
     signal_emit("chat protocol deinit", 1,
 		chat_protocol_find("SILC"));
+#ifdef SILC_DEBUG
+    signal_remove("setup changed", (SIGNAL_FUNC) sig_debug_setup_changed);
+#endif
     
     silc_server_deinit();
     silc_channels_deinit();
     silc_queries_deinit();
+    silc_expandos_deinit();
     
     chat_protocol_unregister("SILC");
     
