@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.46 2002/03/24 14:18:42 priikone Exp $ */
+/* $Id: idlist.c,v 1.55 2002/09/07 15:22:20 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -287,6 +287,7 @@ typedef struct {
   SilcBuffer client_id_list;
   SilcGetClientCallback completion;
   void *context;
+  int res_count;
 } *GetClientsByListInternal;
 
 SILC_CLIENT_CMD_FUNC(get_clients_list_callback)
@@ -301,6 +302,14 @@ SILC_CLIENT_CMD_FUNC(get_clients_list_callback)
 
   SILC_LOG_DEBUG(("Start"));
 
+  if (i->res_count) {
+    i->res_count--;
+    if (i->res_count)
+      return;
+  }
+
+  SILC_LOG_DEBUG(("Resolved all clients"));
+
   for (c = 0; c < i->list_count; c++) {
     SilcUInt16 idp_len;
     SilcClientID *client_id;
@@ -362,6 +371,7 @@ void silc_client_get_clients_by_list(SilcClient client,
   unsigned char **res_argv = NULL;
   SilcUInt32 *res_argv_lens = NULL, *res_argv_types = NULL, res_argc = 0;
   GetClientsByListInternal in;
+  bool wait_res = FALSE;
 
   SILC_LOG_DEBUG(("Start"));
 
@@ -377,6 +387,7 @@ void silc_client_get_clients_by_list(SilcClient client,
     SilcUInt16 idp_len;
     SilcClientID *client_id;
     SilcClientEntry entry;
+    bool ret;
 
     /* Get Client ID */
     SILC_GET16_MSB(idp_len, client_id_list->data + 2);
@@ -388,30 +399,39 @@ void silc_client_get_clients_by_list(SilcClient client,
     }
 
     /* Check if we have this client cached already. */
-    id_cache = NULL;
-    silc_idcache_find_by_id_one_ext(conn->client_cache, (void *)client_id, 
-				    NULL, NULL, 
-				    silc_hash_client_id_compare, NULL,
-				    &id_cache);
+    ret =
+      silc_idcache_find_by_id_one_ext(conn->client_cache, (void *)client_id, 
+				      NULL, NULL, 
+				      silc_hash_client_id_compare, NULL,
+				      &id_cache);
 
     /* If we don't have the entry or it has incomplete info, then resolve
        it from the server. */
-    entry = id_cache ? (SilcClientEntry)id_cache->context : NULL;
-    if (!id_cache || !entry->nickname) {
+    if (!ret || !((SilcClientEntry)id_cache->context)->nickname) {
+      entry = ret ? (SilcClientEntry)id_cache->context : NULL;
 
       if (entry) {
 	if (entry->status & SILC_CLIENT_STATUS_RESOLVING) {
-	  entry->status &= ~SILC_CLIENT_STATUS_RESOLVING;
+	  /* Attach to this resolving and wait until it finishes */
+	  silc_client_command_pending(
+			    conn, SILC_COMMAND_NONE, 
+			    entry->resolve_cmd_ident,
+			    silc_client_command_get_clients_list_callback, 
+			    (void *)in);
+	  wait_res = TRUE;
+	  in->res_count++;
+
 	  silc_free(client_id);
 	  silc_buffer_pull(client_id_list, idp_len);
 	  continue;
 	}
 
 	entry->status |= SILC_CLIENT_STATUS_RESOLVING;
+	entry->resolve_cmd_ident = conn->cmd_ident + 1;
       }
 
       /* No we don't have it, query it from the server. Assemble argument
-	 table that will be sent fr the IDENTIFY command later. */
+	 table that will be sent for the IDENTIFY command later. */
       res_argv = silc_realloc(res_argv, sizeof(*res_argv) *
 			      (res_argc + 1));
       res_argv_lens = silc_realloc(res_argv_lens, sizeof(*res_argv_lens) *
@@ -428,6 +448,9 @@ void silc_client_get_clients_by_list(SilcClient client,
     silc_buffer_pull(client_id_list, idp_len);
   }
 
+  silc_buffer_push(client_id_list, client_id_list->data - 
+		   client_id_list->head);
+
   /* Query the client information from server if the list included clients
      that we don't know about. */
   if (res_argc) {
@@ -450,9 +473,8 @@ void silc_client_get_clients_by_list(SilcClient client,
     silc_client_command_pending(conn, SILC_COMMAND_IDENTIFY, conn->cmd_ident,
 				silc_client_command_get_clients_list_callback, 
 				(void *)in);
+    in->res_count++;
 
-    silc_buffer_push(client_id_list, client_id_list->data - 
-		     client_id_list->head);
     silc_buffer_free(res_cmd);
     silc_free(res_argv);
     silc_free(res_argv_lens);
@@ -460,8 +482,8 @@ void silc_client_get_clients_by_list(SilcClient client,
     return;
   }
 
-  silc_buffer_push(client_id_list, client_id_list->data - 
-		   client_id_list->head);
+  if (wait_res)
+    return;
 
   /* We have the clients in cache, get them and call the completion */
   silc_client_command_get_clients_list_callback((void *)in, NULL);
@@ -506,10 +528,13 @@ SILC_CLIENT_CMD_FUNC(get_client_by_id_callback)
 
   /* Get the client */
   entry = silc_client_get_client_by_id(i->client, i->conn, i->client_id);
-  if (entry)
-    i->completion(i->client, i->conn, &entry, 1, i->context);
-  else
-    i->completion(i->client, i->conn, NULL, 0, i->context);
+  if (entry) {
+    if (i->completion)
+      i->completion(i->client, i->conn, &entry, 1, i->context);
+  } else {
+    if (i->completion)
+      i->completion(i->client, i->conn, NULL, 0, i->context);
+  }
 
   silc_free(i->client_id);
   silc_free(i);
@@ -544,7 +569,7 @@ void silc_client_get_client_by_id_resolve(SilcClient client,
   /* Send the command */
   idp = silc_id_payload_encode(client_id, SILC_ID_CLIENT);
   silc_client_command_send(client, conn, SILC_COMMAND_WHOIS, conn->cmd_ident,
-			   1, 3, idp->data, idp->len);
+			   1, 4, idp->data, idp->len);
   silc_buffer_free(idp);
 
   /* Add pending callback */
@@ -621,9 +646,14 @@ void silc_client_update_client(SilcClient client,
 
   SILC_LOG_DEBUG(("Start"));
 
-  if (!client_entry->username && username)
-    silc_parse_userfqdn(username, &client_entry->username, 
+  if ((!client_entry->username || !client_entry->hostname) && username) {
+    silc_free(client_entry->username);
+    silc_free(client_entry->hostname);
+    client_entry->username = NULL;
+    client_entry->hostname = NULL;
+    silc_parse_userfqdn(username, &client_entry->username,
 			&client_entry->hostname);
+  }
   if (!client_entry->realname && userinfo)
     client_entry->realname = strdup(userinfo);
   if (!client_entry->nickname && nickname) {
@@ -951,6 +981,8 @@ SilcServerEntry silc_client_add_server(SilcClient client,
 {
   SilcServerEntry server_entry;
 
+  SILC_LOG_DEBUG(("Start"));
+
   server_entry = silc_calloc(1, sizeof(*server_entry));
   if (!server_entry || !server_id)
     return NULL;
@@ -987,6 +1019,34 @@ bool silc_client_del_server(SilcClient client, SilcClientConnection conn,
   return ret;
 }
 
+/* Updates the `server_entry' with the new information sent as argument. */
+
+void silc_client_update_server(SilcClient client,
+			       SilcClientConnection conn,
+			       SilcServerEntry server_entry,
+			       const char *server_name,
+			       const char *server_info)
+{
+  SILC_LOG_DEBUG(("Start"));
+
+  if (server_name && (!server_entry->server_name ||
+		      strcmp(server_entry->server_name, server_name))) {
+
+    silc_idcache_del_by_context(conn->server_cache, server_entry);
+    silc_free(server_entry->server_name);
+    server_entry->server_name = strdup(server_name);
+    silc_idcache_add(conn->server_cache, server_entry->server_name,
+		     server_entry->server_id,
+		     server_entry, 0, NULL);
+  }
+
+  if (server_info && (!server_entry->server_info ||
+		      strcmp(server_entry->server_info, server_info))) {
+    silc_free(server_entry->server_info);
+    server_entry->server_info = strdup(server_info);
+  }
+}
+
 /* Formats the nickname of the client specified by the `client_entry'.
    If the format is specified by the application this will format the
    nickname and replace the old nickname in the client entry. If the
@@ -999,6 +1059,7 @@ void silc_client_nickname_format(SilcClient client,
   char *cp;
   char *newnick = NULL;
   int i, off = 0, len;
+  bool freebase;
   SilcClientEntry *clients;
   SilcUInt32 clients_count = 0;
   SilcClientEntry unformatted = NULL;
@@ -1021,10 +1082,15 @@ void silc_client_nickname_format(SilcClient client,
     return;
 
   len = 0;
-  for (i = 0; i < clients_count; i++)
+  freebase = TRUE;
+  for (i = 0; i < clients_count; i++) {
     if (clients[i]->valid && clients[i] != client_entry)
       len++;
-  if (!len)
+    if (clients[i]->valid && clients[i] != client_entry &&
+	!strcmp(clients[i]->nickname, client_entry->nickname))
+      freebase = FALSE;
+  }
+  if (!len || freebase)
     return;
 
   cp = client->internal->params->nickname_format;
@@ -1049,6 +1115,9 @@ void silc_client_nickname_format(SilcClient client,
       if (!client_entry->hostname)
 	break;
       len = strcspn(client_entry->hostname, ".");
+      i = strcspn(client_entry->hostname, "-");
+      if (i < len)
+        len = i;
       newnick = silc_realloc(newnick, sizeof(*newnick) * (off + len));
       memcpy(&newnick[off], client_entry->hostname, len);
       off += len;
