@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: sftp_server.c,v 1.5 2001/11/26 19:05:58 priikone Exp $ */
+/* $Id: sftp_server.c,v 1.8 2002/05/03 17:52:23 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcsftp.h"
@@ -25,7 +25,6 @@
 
 /* SFTP Server context */
 typedef struct {
-  SilcSocketConnection sock;
   SilcSFTPSendPacketCallback send_packet;
   void *send_context;
   SilcSFTPMonitors monitors;
@@ -39,7 +38,7 @@ typedef struct {
 
 static void silc_sftp_send_packet(SilcSFTPServer sftp,
 				  SilcSFTPPacket type, 
-				  uint32 len, ...)
+				  SilcUInt32 len, ...)
 {
   SilcBuffer tmp;
   va_list vp;
@@ -55,7 +54,7 @@ static void silc_sftp_send_packet(SilcSFTPServer sftp,
 		   sftp->packet->len);
 
   /* Send the packet */
-  (*sftp->send_packet)(sftp->sock, sftp->packet, sftp->send_context);
+  (*sftp->send_packet)(sftp->packet, sftp->send_context);
 
   /* Clear packet */
   sftp->packet->data = sftp->packet->tail = sftp->packet->head;
@@ -66,7 +65,7 @@ static void silc_sftp_send_packet(SilcSFTPServer sftp,
 
 static void silc_sftp_send_error(SilcSFTPServer sftp,
 				 SilcSFTPStatus status,
-				 uint32 id)
+				 SilcUInt32 id)
 {
   SILC_LOG_DEBUG(("Send error %d", status));
 
@@ -87,7 +86,7 @@ static void silc_sftp_server_status(SilcSFTP sftp,
 				    void *context)
 {
   SilcSFTPServer server = (SilcSFTPServer)sftp;
-  uint32 id = (uint32)context;
+  SilcUInt32 id = (SilcUInt32)context;
   int mlen, llen;
 
   SILC_LOG_DEBUG(("Status callback"));
@@ -118,9 +117,9 @@ static void silc_sftp_server_handle(SilcSFTP sftp,
 				    void *context)
 {
   SilcSFTPServer server = (SilcSFTPServer)sftp;
-  uint32 id = (uint32)context;
+  SilcUInt32 id = (SilcUInt32)context;
   unsigned char *hdata;
-  uint32 hdata_len;
+  SilcUInt32 hdata_len;
 
   SILC_LOG_DEBUG(("Handle callback"));
   SILC_LOG_DEBUG(("Request ID: %d", id));
@@ -149,11 +148,11 @@ static void silc_sftp_server_handle(SilcSFTP sftp,
 static void silc_sftp_server_data(SilcSFTP sftp,
 				  SilcSFTPStatus status,
 				  const unsigned char *data,
-				  uint32 data_len,
+				  SilcUInt32 data_len,
 				  void *context)
 {
   SilcSFTPServer server = (SilcSFTPServer)sftp;
-  uint32 id = (uint32)context;
+  SilcUInt32 id = (SilcUInt32)context;
 
   SILC_LOG_DEBUG(("Data callback"));
   SILC_LOG_DEBUG(("Request ID: %d", id));
@@ -178,7 +177,7 @@ static void silc_sftp_server_name(SilcSFTP sftp,
 				  void *context)
 {
   SilcSFTPServer server = (SilcSFTPServer)sftp;
-  uint32 id = (uint32)context;
+  SilcUInt32 id = (SilcUInt32)context;
   SilcBuffer namebuf;
 
   SILC_LOG_DEBUG(("Name callback"));
@@ -209,7 +208,7 @@ static void silc_sftp_server_attr(SilcSFTP sftp,
 				  void *context)
 {
   SilcSFTPServer server = (SilcSFTPServer)sftp;
-  uint32 id = (uint32)context;
+  SilcUInt32 id = (SilcUInt32)context;
   SilcBuffer attr_buf;
 
   SILC_LOG_DEBUG(("Attr callback"));
@@ -221,6 +220,10 @@ static void silc_sftp_server_attr(SilcSFTP sftp,
   }
 
   attr_buf = silc_sftp_attr_encode(attrs);
+  if (!attr_buf) {
+    silc_sftp_send_error(server, SILC_SFTP_STATUS_FAILURE, id);
+    return;
+  }
 
   silc_sftp_send_packet(server, SILC_SFTP_ATTRS, 4 + attr_buf->len,
 			SILC_STR_UI_INT(id),
@@ -235,11 +238,11 @@ static void silc_sftp_server_attr(SilcSFTP sftp,
 static void silc_sftp_server_extended(SilcSFTP sftp,
 				      SilcSFTPStatus status,
 				      const unsigned char *data,
-				      uint32 data_len,
+				      SilcUInt32 data_len,
 				      void *context)
 {
   SilcSFTPServer server = (SilcSFTPServer)sftp;
-  uint32 id = (uint32)context;
+  SilcUInt32 id = (SilcUInt32)context;
 
   SILC_LOG_DEBUG(("Extended callback"));
   SILC_LOG_DEBUG(("Request ID: %d", id));
@@ -255,21 +258,20 @@ static void silc_sftp_server_extended(SilcSFTP sftp,
 			SILC_STR_END);
 }
 
-/* Starts SFTP server by associating the socket connection `sock' to the
-   created SFTP server context.  This function returns the allocated
-   SFTP client context or NULL on error. The `send_packet' is called
+/* Starts SFTP server and returns context to it.  This function returns the
+   allocated SFTP client context or NULL on error. The `send_packet' is called
    by the library when it needs to send a packet. The `fs' is the
    structure containing filesystem access callbacks. */
 
-SilcSFTP silc_sftp_server_start(SilcSocketConnection sock,
-				SilcSFTPSendPacketCallback send_packet,
+SilcSFTP silc_sftp_server_start(SilcSFTPSendPacketCallback send_packet,
 				void *send_context,
 				SilcSFTPFilesystem fs)
 {
   SilcSFTPServer server;
 
   server = silc_calloc(1, sizeof(*server));
-  server->sock = sock;
+  if (!server)
+    return NULL;
   server->send_packet = send_packet;
   server->send_context = send_context;
   server->fs = fs;
@@ -319,10 +321,10 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   SilcSFTPPacket type;
   char *filename = NULL, *path = NULL;
   const unsigned char *payload = NULL;
-  uint32 payload_len;
+  SilcUInt32 payload_len;
   int ret;
   SilcBufferStruct buf;
-  uint32 id;
+  SilcUInt32 id;
   SilcSFTPAttributes attrs;
   SilcSFTPHandle handle;
   SilcSFTPMonitorDataStruct mdata;
@@ -369,7 +371,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
     {
       SilcSFTPFileOperation pflags;
       unsigned char *attr_buf;
-      uint32 attr_len = 0;
+      SilcUInt32 attr_len = 0;
       SilcBufferStruct tmpbuf;
 
       SILC_LOG_DEBUG(("Open request"));
@@ -387,8 +389,12 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
       if (attr_len) {
 	silc_buffer_set(&tmpbuf, attr_buf, attr_len);
 	attrs = silc_sftp_attr_decode(&tmpbuf);
+	if (!attrs)
+	  goto failure;
       } else {
 	attrs = silc_calloc(1, sizeof(*attrs));
+	if (!attrs)
+	  goto failure;
       }
 
       /* Call monitor */
@@ -411,7 +417,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_CLOSE:
     {
       unsigned char *hdata;
-      uint32 hdata_len;
+      SilcUInt32 hdata_len;
 
       SILC_LOG_DEBUG(("Close request"));
 
@@ -447,9 +453,9 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_READ:
     {
       unsigned char *hdata;
-      uint32 hdata_len;
-      uint64 offset;
-      uint32 len;
+      SilcUInt32 hdata_len;
+      SilcUInt64 offset;
+      SilcUInt32 len;
 
       SILC_LOG_DEBUG(("Read request"));
 
@@ -490,10 +496,10 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_WRITE:
     {
       unsigned char *hdata;
-      uint32 hdata_len;
-      uint64 offset;
+      SilcUInt32 hdata_len;
+      SilcUInt64 offset;
       unsigned char *data;
-      uint32 data_len;
+      SilcUInt32 data_len;
 
       SILC_LOG_DEBUG(("Read request"));
 
@@ -593,7 +599,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_MKDIR:
     {
       unsigned char *attr_buf;
-      uint32 attr_len = 0;
+      SilcUInt32 attr_len = 0;
       SilcBufferStruct tmpbuf;
 
       SILC_LOG_DEBUG(("Mkdir request"));
@@ -610,8 +616,12 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
       if (attr_len) {
 	silc_buffer_set(&tmpbuf, attr_buf, attr_len);
 	attrs = silc_sftp_attr_decode(&tmpbuf);
+	if (!attrs)
+	  goto failure;
       } else {
 	attrs = silc_calloc(1, sizeof(*attrs));
+	if (!attrs)
+	  goto failure;
       }
 
       /* Call monitor */
@@ -685,7 +695,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_READDIR:
     {
       unsigned char *hdata;
-      uint32 hdata_len;
+      SilcUInt32 hdata_len;
 
       SILC_LOG_DEBUG(("Readdir request"));
 
@@ -773,7 +783,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_FSTAT:
     {
       unsigned char *hdata;
-      uint32 hdata_len;
+      SilcUInt32 hdata_len;
 
       SILC_LOG_DEBUG(("Fstat request"));
 
@@ -809,7 +819,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_SETSTAT:
     {
       unsigned char *attr_buf;
-      uint32 attr_len = 0;
+      SilcUInt32 attr_len = 0;
       SilcBufferStruct tmpbuf;
 
       SILC_LOG_DEBUG(("Setstat request"));
@@ -826,8 +836,12 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
       if (attr_len) {
 	silc_buffer_set(&tmpbuf, attr_buf, attr_len);
 	attrs = silc_sftp_attr_decode(&tmpbuf);
+	if (!attrs)
+	  goto failure;
       } else {
 	attrs = silc_calloc(1, sizeof(*attrs));
+	if (!attrs)
+	  goto failure;
       }
 
       /* Call monitor */
@@ -849,7 +863,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
   case SILC_SFTP_FSETSTAT:
     {
       unsigned char *hdata, *attr_buf;
-      uint32 hdata_len, attr_len = 0;
+      SilcUInt32 hdata_len, attr_len = 0;
       SilcBufferStruct tmpbuf;
 
       SILC_LOG_DEBUG(("Fsetstat request"));
@@ -867,8 +881,12 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
       if (attr_len) {
 	silc_buffer_set(&tmpbuf, attr_buf, attr_len);
 	attrs = silc_sftp_attr_decode(&tmpbuf);
+	if (!attrs)
+	  goto failure;
       } else {
 	attrs = silc_calloc(1, sizeof(*attrs));
+	if (!attrs)
+	  goto failure;
       }
 
       /* Get the handle */
@@ -982,7 +1000,7 @@ void silc_sftp_server_receive_process(SilcSFTP sftp,
     {
       char *request = NULL;
       unsigned char *data;
-      uint32 data_len;
+      SilcUInt32 data_len;
 
       SILC_LOG_DEBUG(("Extended request"));
 
