@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcconfig.c,v 1.8 2002/02/24 11:14:42 priikone Exp $ */
+/* $Id: silcconfig.c,v 1.13 2002/06/25 21:43:18 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -60,6 +60,7 @@ struct SilcConfigEntityObject {
 static char *errorstrs[] = {
   "-OK",					/* SILC_CONFIG_OK */
   "-SILENT",					/* SILC_CONFIG_ESILENT */
+  "-PRINTLINE",					/* SILC_CONFIG_EPRINTLINE */
   "Invalid syntax",				/* SILC_CONFIG_EGENERIC */
   "Internal error! Please report this bug",	/* SILC_CONFIG_EINTERNAL */
   "Can't open specified file",			/* SILC_CONFIG_ECANTOPEN */
@@ -250,7 +251,7 @@ static void *silc_config_marshall(SilcConfigType type, const char *val)
 /* Tries to open the config file and returns a valid SilcConfigFile object
  * or NULL if failed */
 
-SilcConfigFile *silc_config_open(char *configfile)
+SilcConfigFile *silc_config_open(const char *configfile)
 {
   char *buffer;
   SilcUInt32 filelen;
@@ -272,14 +273,6 @@ SilcConfigFile *silc_config_open(char *configfile)
 void silc_config_close(SilcConfigFile *file)
 {
   if (file) {
-    /* XXX FIXME: this check could probably be removed later */
-    SilcUInt32 my_len = (SilcUInt32) (strchr(file->base, EOF) - file->base);
-    SILC_CONFIG_DEBUG(("file=0x%x name=\"%s\" level=%d line=%lu", (SilcUInt32) file,
-			file->filename, file->level, file->line));
-    if (my_len != file->len) {
-      fprintf(stderr, "FATAL ERROR: saved len and current len does not match!\n");
-      abort();
-    }
     silc_free(file->filename);
     memset(file->base, 'F', file->len);
     silc_free(file->base);
@@ -363,11 +356,16 @@ char *silc_config_read_current_line(SilcConfigFile *file)
 
 /* (Private) destroy a SilcConfigEntity */
 
-static void silc_config_destroy(SilcConfigEntity ent)
+static void silc_config_destroy(SilcConfigEntity ent, bool destroy_opts)
 {
   SilcConfigOption *oldopt, *nextopt;
   SILC_CONFIG_DEBUG(("Freeing config entity [ent=0x%x] [opts=0x%x]",
 			(SilcUInt32) ent, (SilcUInt32) ent->opts));
+
+  /* if she wants to preserve options just free the object struct */
+  if (!destroy_opts)
+    goto skip_sect;
+
   for (oldopt = ent->opts; oldopt; oldopt = nextopt) {
     nextopt = oldopt->next;
     memset(oldopt->name, 'F', strlen(oldopt->name) + 1);
@@ -375,6 +373,8 @@ static void silc_config_destroy(SilcConfigEntity ent)
     memset(oldopt, 'F', sizeof(*oldopt));
     silc_free(oldopt);
   }
+
+ skip_sect:
   memset(ent, 'F', sizeof(*ent));
   silc_free(ent);
 }
@@ -641,7 +641,9 @@ int silc_config_main(SilcConfigEntity ent)
    * destroyed. */
  main_cleanup:
   if ((file->level != 0) || (file->included != TRUE))
-    silc_config_destroy(ent);
+    silc_config_destroy(ent, TRUE);
+  else
+    silc_config_destroy(ent, FALSE);
 
  main_end:
   return ret;
