@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silclog.c,v 1.20 2002/03/17 14:00:30 priikone Exp $ */
+/* $Id: silclog.c,v 1.28 2002/09/02 21:09:11 johnny Exp $ */
 
 #include "silcincludes.h"
 
@@ -30,10 +30,10 @@
 /* Our working struct -- at the moment we keep it private, but this could
  * change in the future */
 struct SilcLogStruct {
-  char *filename;
+  char filename[256];
   FILE *fp;
   SilcUInt32 maxsize;
-  char *typename;
+  const char *typename;
   SilcLogType type;
   SilcLogCb cb;
   void *context;
@@ -43,12 +43,15 @@ typedef struct SilcLogStruct *SilcLog;
 /* These are the known logging channels.  We initialize this struct with most
  * of the fields set to NULL, because we'll fill in those values at runtime. */
 static struct SilcLogStruct silclogs[SILC_LOG_MAX] = {
-  {NULL, NULL, 0, "Info", SILC_LOG_INFO, NULL, NULL},
-  {NULL, NULL, 0, "Warning", SILC_LOG_WARNING, NULL, NULL},
-  {NULL, NULL, 0, "Error", SILC_LOG_ERROR, NULL, NULL},
-  {NULL, NULL, 0, "Fatal", SILC_LOG_FATAL, NULL, NULL},
+  {"", NULL, 0, "Info", SILC_LOG_INFO, NULL, NULL},
+  {"", NULL, 0, "Warning", SILC_LOG_WARNING, NULL, NULL},
+  {"", NULL, 0, "Error", SILC_LOG_ERROR, NULL, NULL},
+  {"", NULL, 0, "Fatal", SILC_LOG_FATAL, NULL, NULL},
 };
 
+/* Causes logging output to contain timestamps */
+bool silc_log_timestamp = TRUE;
+
 /* If TRUE, log files will be flushed for each log input */
 bool silc_log_quick = FALSE;
 
@@ -60,7 +63,7 @@ bool silc_debug_hexdump = FALSE;
 long silc_log_flushdelay = 300;
 
 /* Regular pattern matching expression for the debug output */
-char *silc_log_debug_string = NULL;
+char silc_log_debug_string[128];
 
 /* Debug callbacks. If set, these are triggered for each specific output. */
 static SilcLogDebugCb silc_log_debug_cb = NULL;
@@ -96,7 +99,7 @@ static SilcLog silc_log_find_by_type(SilcLogType type)
 }
 
 /* Given an open log file, checks the size and rotates it if there is a
- * max size set less then the current size */
+ * max size set lower then the current size */
 static void silc_log_checksize(SilcLog log)
 {
   char newname[127];
@@ -104,12 +107,13 @@ static void silc_log_checksize(SilcLog log)
 
   if (!log || !log->fp || !log->maxsize)
     return; /* we are not interested */
+
   if ((size = ftell(log->fp)) < 0) {
     /* OMG, EBADF is here.. we'll try our best.. */
     FILE *oldfp = log->fp;
     fclose(oldfp); /* we can discard the error */
     log->fp = NULL; /* make sure we don't get here recursively */
-    SILC_LOG_ERROR(("Error while checking size of the log file %s, fp=%d",
+    SILC_LOG_ERROR(("Error while checking size of the log file %s, fp=%p",
 		    log->filename, oldfp));
     return;
   }
@@ -142,12 +146,13 @@ static bool silc_log_reset(SilcLog log)
     fflush(log->fp);
     fclose(log->fp);
   }
-  if (!log->filename) return FALSE;
+  if (!log->filename[0]) return FALSE;
   if (!(log->fp = fopen(log->filename, "a+"))) {
     SILC_LOG_WARNING(("Couldn't reset logfile %s for type \"%s\": %s",
 		      log->filename, log->typename, strerror(errno)));
     return FALSE;
   }
+
   return TRUE;
 }
 
@@ -176,7 +181,7 @@ SILC_TASK_CALLBACK(silc_log_fflush_callback)
 
 void silc_log_output(SilcLogType type, char *string)
 {
-  char *typename = NULL;
+  const char *typename = NULL;
   FILE *fp;
   SilcLog log;
 
@@ -218,10 +223,15 @@ void silc_log_output(SilcLogType type, char *string)
   goto end;
 
  found:
-  fprintf(fp, "[%s] [%s] %s\n", silc_get_time(), typename, string);
+  /* writes the logging string to the selected channel */
+  if (silc_log_timestamp)
+    fprintf(fp, "[%s] [%s] %s\n", silc_get_time(), typename, string);
+  else
+    fprintf(fp, "[%s] %s\n", typename, string);
+
   if (silc_log_quick || silc_log_starting) {
     fflush(fp);
-    if (log)
+    if (log) /* we may have been redirected to stderr */
       silc_log_checksize(log);
   }
 
@@ -254,7 +264,7 @@ bool silc_log_set_file(SilcLogType type, char *filename, SilcUInt32 maxsize,
   SilcLog log;
 
   log = silc_log_find_by_type(type);
-  if (!log || !scheduler)
+  if (!log)
     return FALSE;
 
   SILC_LOG_DEBUG(("Setting \"%s\" file to %s (max size=%d)",
@@ -269,32 +279,34 @@ bool silc_log_set_file(SilcLogType type, char *filename, SilcUInt32 maxsize,
     }
   }
 
-  /* remove old file */
-  if (log->filename) {
-    if (log->fp) {
-      fflush(fp);
-      fclose(fp);
-    }
-    silc_free(log->filename);
-    log->filename = NULL;
+  /* clean the logging channel */
+  if (strlen(log->filename)) {
+    if (log->fp)
+      fclose(log->fp);
+    memset(log->filename, 0, sizeof(log->filename));
     log->fp = NULL;
   }
 
   if (fp) {
-    log->filename = strdup(filename);
+    memset(log->filename, 0, sizeof(log->filename));
+    strncpy(log->filename, filename,
+	    strlen(filename) < sizeof(log->filename) ? strlen(filename) :
+	    sizeof(log->filename) - 1);
     log->fp = fp;
     log->maxsize = maxsize;
   }
 
-  if (silc_log_scheduled)
-    return TRUE;
+  if (scheduler) {
+    if (silc_log_scheduled)
+      return TRUE;
 
-  /* add schedule hook with a short delay to make sure we'll use right delay */
-  silc_schedule_task_add(scheduler, 0, silc_log_fflush_callback,
-			 (void *) scheduler, 10, 0,
-			 SILC_TASK_TIMEOUT, SILC_TASK_PRI_NORMAL);
-
-  silc_log_scheduled = TRUE;
+    /* Add schedule hook with a short delay to make sure we'll use 
+       right delay */
+    silc_schedule_task_add(scheduler, 0, silc_log_fflush_callback,
+			   (void *) scheduler, 10, 0,
+			   SILC_TASK_TIMEOUT, SILC_TASK_PRI_NORMAL);
+    silc_log_scheduled = TRUE;
+  }
 
   return TRUE;
 }
@@ -483,10 +495,17 @@ void silc_log_reset_debug_callbacks()
 
 void silc_log_set_debug_string(const char *debug_string)
 {
-  silc_free(silc_log_debug_string);
+  char *string;
+  int len;
   if ((strchr(debug_string, '(') && strchr(debug_string, ')')) ||
       strchr(debug_string, '$'))
-    silc_log_debug_string = strdup(debug_string);
+    string = strdup(debug_string);
   else
-    silc_log_debug_string = silc_string_regexify(debug_string);
+    string = silc_string_regexify(debug_string);
+  len = strlen(string);
+  if (len >= sizeof(silc_log_debug_string))
+    len = sizeof(silc_log_debug_string) - 1;
+  memset(silc_log_debug_string, 0, sizeof(silc_log_debug_string));
+  strncpy(silc_log_debug_string, string, len);
+  silc_free(string);
 }
